/*
 * main.c
 *
 * Let the LED blink with a frequency passed as parameter.
 *
 * This example is an advanced training material for experienced students.
 *
 * It is not necessary to implement one function per task. A function can be used by
 * several tasks. Parameters tells it what to do. The parameters have to be passed by
 * a pointer. If more than one parameter is needed a struct is used.
 *
 * Created: 2018-07-23
 *  Author: Wolfgang Neff
 */

#include <avr/io.h>
#include <FreeRTOS.h>
#include <task.h>

#include "board.h"

#define TASK_PRIORITY (tskIDLE_PRIORITY+1)

// We want to pass two bits of information:
// - The bitmap of the LED.
// - The ticks to delay.
typedef struct {
	uint8_t led;
	TickType_t delay;
} led_params_t;

// We define the parameters for each LED.
led_params_t led_params = { LED_PIN_bm, pdMS_TO_TICKS(250) };

void vLEDTask(void* pvParameters)
{
	// Parameters are void*. This means they are anonymous.
	// We have to give them the correct data type in order
	// to be able to access the components of the struct.
	led_params_t* params = (led_params_t*)pvParameters;
	// Initialize the hardware of the task.
	LED_PORT.DIRSET = params->led;
	LED_PORT.OUTSET = params->led;
	// Super loop of the task.
	while (1) {
		// Toggle the LED.
		LED_PORT.OUTTGL = params->led;
		// Wait a bit.
		vTaskDelay(params->delay);
	}
}

int main(void)
{
	// Create the tasks.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, &led_params, TASK_PRIORITY, NULL);
	
	// Start the scheduler.
	vTaskStartScheduler();
}
