/*
 * main.c
 *
 * This example is an advanced training material for experienced students.
 *
 * Polling buttons is a wast of performance. Interrupts can be used instead. Doing
 * so is much more difficult, however. Special kinds of routines have to be used
 * and care for interrupts with a higher priority has to be taken.
 *
 * Please see www.freertos.org for more information.
 * xSemaphoreGiveFromISR: FreeRTOS/API Reference/Semaphore / Mutexes
 *
 * Created: 2018-07-23
 *   Author: Wolfgang Neff
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <FreeRTOS.h>
#include <semphr.h>
#include <task.h>

#include "board.h"

#define TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define LED_DELAY pdMS_TO_TICKS(1000)

SemaphoreHandle_t xSemaphore;

void vLEDTask(void* pvParameters)
{
	// Initialize the LED.
    LED_PORT.DIR = LED_PIN_bm;
    LED_PORT.OUT = LED_PIN_bm;
	// Super loop of the task.
	while (1) {
		xSemaphoreTake(xSemaphore, portMAX_DELAY);
		LED_PORT.OUTCLR = LED_PIN_bm;
		vTaskDelay(LED_DELAY);
		LED_PORT.OUTSET = LED_PIN_bm;
	}
}

int main(void)
{
    // Initialize button and enable port interrupts.
	BUTTON_PINCTRL = PORT_PULLUPEN_bm | PORT_ISC_FALLING_gc;

    // Create the binary semaphore.
	xSemaphore = xSemaphoreCreateBinary();

	// Create the task.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, TASK_PRIORITY, NULL);

	// Start the scheduler.
	vTaskStartScheduler();
}

// Interrupt service routine for INT0.
ISR(PORTB_PORT_vect)
{
    // See FreeRTOS/API Reference/Semaphore / Mutexes/xSemaphoreGiveFromISR
	// for further information. 
	BaseType_t xHigherPriorityTaskWoken = pdFALSE;
	xSemaphoreGiveFromISR(xSemaphore, &xHigherPriorityTaskWoken);
	BUTTON_PORT.INTFLAGS |= BUTTON_PIN_bm;
	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}

