/** \file switch.h
 *
 * \brief Debounce the push-button.
 *
 * This module provides routines to debounce the mechanical active
 * low  push-button.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-17
 *
 * \par History
 *      Created: 2018-07-17
 */

#ifndef SWITCH_H_
#define SWITCH_H_

#include "board.h"
#include <stdint.h>

#define SWITCH_PRESSED() debounce(&BUTTON_PORT,BUTTON_PIN_bm,0)

#define SWITCH0_PRESSED() debounce(&BUTTON0_PORT,BUTTON0_PIN_bm,0)
#define SWITCH1_PRESSED() debounce(&BUTTON1_PORT,BUTTON1_PIN_bm,1)

#ifdef __cplusplus
extern "C"
{
#endif

/// <summary>Debounces active low push-buttons.</summary>
/// <remarks>
/// This routine debounces up to eight active low push-buttons. Each key
/// needs an arbitrary key number. This routine is not thread save.
/// </remarks>
/// <param name="port">Port of the key.</param>
/// <param name="mask">Bit mask of the key.</param>
/// <param name="key">Number assigned to the key.</param>
/// <returns>True if key is pressed.</returns>
uint8_t debounce(volatile PORT_t* port, uint8_t mask, uint8_t key);

#ifdef __cplusplus
}
#endif

#endif /* SWITCH_H_ */