/*
 * main.c
 *
 * This example is an advanced training material for experienced students.
 *
 * It is not necessary to create a task to have something done on a regular base.
 * Software timers can be used instead. They are more lightweight than tasks.
 * Software timers can also be use for one time actions in the future which
 * can only be realized difficultly with tasks.
 *
 * Please see www.freertos.org for more information.
 * xTimerCreate: FreeRTOS/API Reference/Software Timers
 * xTimerStart: FreeRTOS/API Reference/Software Timers
 *
 * Modified: 2018-07-23
 *  Author: Wolfgang Neff
 */ 

#include <avr/io.h>
#include <FreeRTOS.h>
#include <timers.h>

#include "board.h"

#define LED_DELAY pdMS_TO_TICKS(250)

// Create the timer.
TimerHandle_t xTimer;

// This timer callback function lets blink LED1
void vTimerCallback(TimerHandle_t pxTimer)
{
	LED_PORT.OUTTGL = LED_PIN_bm;
}

int main(void)
{
	// Setup the timer controlled LED.
	LED_PORT.DIR = LED_PIN_bm;
	LED_PORT.OUT = LED_PIN_bm;
	
    // Create a timer with the name "Timer". Call vTimerCallback after
	// LED_DELAY ticks. Reload timer when it expires. The callback
	// function is thus called every LED_DELAY ticks.
	xTimer = xTimerCreate("Timer", LED_DELAY, pdTRUE, NULL, vTimerCallback);
	xTimerStart(xTimer, 0);

	// Start the scheduler. The idle task is always there.
	vTaskStartScheduler();
}
